'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (_ref) {
  var t = _ref.types;

  return {
    visitor: {
      NewExpression: function NewExpression(path) {
        maybeReplaceWithRegExpLiteral(path, t);
      },
      CallExpression: function CallExpression(path) {
        // equivalent to `new RegExp()` according to §21.2.3
        maybeReplaceWithRegExpLiteral(path, t);
      },
      RegExpLiteral: function RegExpLiteral(path) {
        var node = path.node;


        var xregexp = (0, _xregexp2.default)(node.pattern, node.flags);
        var source = xregexp.source,
            flags = xregexp.flags;


        if (source !== node.pattern || flags !== node.flags) {
          path.replaceWith(t.RegExpLiteral(source, flags));
        }
      },
      MemberExpression: function MemberExpression(path) {
        var _path$node = path.node,
            object = _path$node.object,
            property = _path$node.property;


        if (object.type === 'RegExpLiteral' && property.name === 'source') {
          path.replaceWith(t.StringLiteral(object.pattern));
        }
      }
    }
  };
};

var _xregexp = require('xregexp');

var _xregexp2 = _interopRequireDefault(_xregexp);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function createRegExpLiteral(args, t) {
  var evaluatedArgs = args.map(function (a) {
    return a.evaluate();
  });
  if (!evaluatedArgs.every(function (a) {
    return a.confident && typeof a.value === 'string';
  })) {
    return;
  }
  var pattern = evaluatedArgs.length >= 1 && evaluatedArgs[0].value !== '' ? evaluatedArgs[0].value : '(?:)';
  var flags = evaluatedArgs.length >= 2 ? evaluatedArgs[1].value : '';

  var xregexp = new _xregexp2.default(pattern, flags);

  return t.regExpLiteral(xregexp.source, xregexp.flags);
}

function maybeReplaceWithRegExpLiteral(path, t) {
  if (!t.isIdentifier(path.node.callee, { name: 'RegExp' })) {
    return;
  }
  var regExpLiteral = createRegExpLiteral(path.get('arguments'), t);
  if (regExpLiteral) {
    path.replaceWith(regExpLiteral);
  }
}